#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ezedcfg` package."""

import pytest
from yaml.scanner import ScannerError
from yaml.constructor import ConstructorError
from ezedcfg import EZedCfg, UnrecognizedFormatError

# Before python 3.5 json.decoder raised an IOError rather than a JSONDecodeError
pre_35 = False
try:
    from json.decoder import JSONDecodeError
except ImportError:
    pre_35 = True

@pytest.fixture()
def setup_valid_yaml(tmpdir):
    """Returns a list containing three Vector instances"""
    test_config_data = """
    test: 1
    test2: 2
    """

    default_dict = {'test': 'testing', 'test2': 'still testing', 'test3': 123}

    test_cfg = tmpdir.mkdir("sub").join('test_config.yml')
    test_cfg.write(test_config_data)

    ez = EZedCfg(default_dict, str(test_cfg))

    return ez


@pytest.fixture()
def setup_valid_json(tmpdir):
    """Returns a list containing three Vector instances"""
    test_config_data = """
    {"test": 1, "test2": 2}
    """

    default_dict = {'test': 'testing', 'test2': 'still testing', 'test3': 123}

    test_cfg = tmpdir.mkdir("sub").join('test_config.yml')
    test_cfg.write(test_config_data)

    ez = EZedCfg(default_dict, str(test_cfg), file_format='json')

    return ez


@pytest.fixture()
def setup_invalid_yaml(tmpdir):
    """Returns a list containing three Vector instances"""
    test_config_data = """
    test: 1
    test2:
    1
    """

    default_dict = {'test': 'testing', 'test2': 'still testing'}

    test_cfg = tmpdir.mkdir("sub").join('test_config.yml')
    test_cfg.write(test_config_data)

    ez = EZedCfg(default_dict, str(test_cfg))

    return ez

@pytest.fixture()
def setup_dangerous_yaml(tmpdir):
    """Returns a list containing three Vector instances"""
    test_config_data = """
    test: 1
    test2: !!python/object/apply:subprocess.check_output ['ls']
    """

    default_dict = {'test': 'testing', 'test2': 'still testing', 'test3': 123}

    test_cfg = tmpdir.mkdir("sub").join('test_config.yml')
    test_cfg.write(test_config_data)

    ez = EZedCfg(default_dict, str(test_cfg))

    return ez

@pytest.fixture()
def setup_invalid_json(tmpdir):
    """Returns a list containing three Vector instances"""
    test_config_data = """
    {"test": 1, "test2": 2
    """

    default_dict = {'test': 'testing', 'test2': 'still testing', 'test3': 123}

    test_cfg = tmpdir.mkdir("sub").join('test_config.yml')
    test_cfg.write(test_config_data)

    ez = EZedCfg(default_dict, str(test_cfg), file_format='json')

    return ez


def test_initialise():
    ez = EZedCfg({'this-is-a-dict': 'yup'}, 'this/is/the/custom/config/path')
    assert ez.default_config == {
        'this-is-a-dict': 'yup'} and ez.config_path == 'this/is/the/custom/config/path'


def test_read(setup_valid_yaml):
    ez = setup_valid_yaml
    ez.read()
    assert ez.custom_config == {'test': 1, 'test2': 2}


def test_read_json(setup_valid_json):
    ez = setup_valid_json
    ez.read()
    assert ez.custom_config == {'test': 1, 'test2': 2}


def test_merge(setup_valid_yaml):
    ez = setup_valid_yaml
    ez.load()
    assert ez.merged_config == {'test': 1, 'test2': 2, "test3": 123}


def test_merge(setup_valid_json):
    ez = setup_valid_json
    ez.load()
    assert ez.merged_config == {'test': 1, 'test2': 2, "test3": 123}


def test_bad_format_param(setup_valid_yaml):
    with pytest.raises(UnrecognizedFormatError):
        ez = setup_valid_yaml
        ez.file_format = 'xml'
        ez.read()


def test_invalid_yaml(setup_invalid_yaml):
    with pytest.raises(ScannerError):
        ez = setup_invalid_yaml
        ez.load()

def test_dangerous_yaml(setup_dangerous_yaml):
    with pytest.raises(ConstructorError):
        ez = setup_dangerous_yaml
        ez.load()

def test_invalid_json(setup_invalid_json):
    if pre_35:
        with pytest.raises(ValueError):
            ez = setup_invalid_json
            ez.load()
    else:
        with pytest.raises(JSONDecodeError):
            ez = setup_invalid_json
            ez.load()




# -*- coding: utf-8 -*-

"""Main module."""
import logging
import json
import yaml


class EZedCfg:
    def __init__(self, default_config, config_path, file_format='yaml'):
        """

        Args:
            default_config: (dict) A dictionary containing the default
                configuration values.
            config_path: (string) A string containing the path (abs or relative)
                to the desired custom config file.
            file_format: (string) String indicating whether a file is JSON
                (json) or YAML (yaml) formatted
        """
        self.default_config = default_config
        self.config_path = config_path
        self.file_format = file_format.lower().strip()
        self.merged_config = False
        self.custom_config = False

    def read(self):
        """This method attempts to load the file supplied in the config_path parameter.

        Raises:
            UnrecognizedFormatError: If an unknown file format is specified in
            the file_format parameter

        Returns:
            dict: A dict containing the data in the supplied configuration file.

        """

        with open(self.config_path, 'r') as config_file:
            config_string = config_file.read()

        if self.file_format == 'json':
            self.custom_config = json.loads(config_string)
        elif self.file_format == 'yaml':
            self.custom_config = yaml.safe_load(config_string)
        else:
            raise UnrecognizedFormatError(
                'File format ({}) not recognised'.format(str(self.file_format)))

        return self.custom_config

    def merge(self):
        """This method merges the default configuration and the supplied configuration

        Returns:
            dict:  A dict containing the updated configuration.

        """
        if self.custom_config is not False:
            # Retain the unchanged default config.
            self.merged_config = self.default_config
            return self.merged_config.update(self.custom_config)

    def load(self):
        """This method runs the entire file reading and dictionary merging process

        This is the primary entry point for this module as it returns the merged
        configuration.

        Returns:
            dict: A dictionary containing the merged configuration data.

        """

        self.read()
        self.merge()
        return self.merged_config


class UnrecognizedFormatError(Exception):
    """A simple exception raised when an unrecognized encoding
    format is supplied.
    """
    def __init__(self, msg=None):
        if msg is None:
            msg = "Unknown file formatting."
